/* eslint-disable max-len */
import { Injectable } from '@angular/core';
import { DataService } from '../data/data.service';
import { ImageDataService } from '../image-data/image-data.service';
import { Observable, of } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { Indexable } from '../../interfaces/indexable';
import { UiEntry } from '../../interfaces/ui/ui-entry';

/**
 * Mock data connection service.
 * ~extends {@link DataService}
 *
 * Supplies test data.
 */
@Injectable({
  providedIn: 'root'
})
export class MockDataService extends DataService {
  /** Mock data. */
  public readonly mockData = {
    // tslint:disable-next-line: max-line-length
    frequencies: [['Developer', { Count: 16, Percentage: 48, Lightness: 0, Label: 'Developer' }], ['Programmer', { Count: 5, Percentage: 15, Lightness: 37, Label: 'Programmer' }]], languages: [{ Language: 'English', Level: 'Full professional proficiency', Score: 4, Share: 30, Label: 'English' }], projects: [{ Id: 1, From: 43243, To: 61543, 'From Year': 2018, 'From Month': 5, 'Imported Name': 'Pluralsight Skill IQ in partnership with Stack Overflow', 'Months total': 601, 'Duration total': '601', Name: 'Pluralsight Skill IQ in partnership with Stack Overflow', Start: 2018.4166666666667, 'Years total': 50.083333333333336, Type: 'Certification', Color: '#7F00FFC0' }, { Id: 1, From: 43221, To: 43252, 'From Year': 2018, 'From Month': 5, 'Imported Name': 'Life Store (freelance)', 'Months total': 1, 'Duration total': '1 month', Name: 'Life Store (freelance)', Start: 2018.4166666666667, 'Years total': 0.08333333333333333, Type: 'Project', Color: '#004000C0' }], filteredProjects: [{ Id: 1, From: 43221, To: 43252, 'From Year': 2018, 'From Month': 5, 'Imported Name': 'Life Store (freelance)', 'Months total': 1, 'Duration total': '1 month', Name: 'Life Store (freelance)', Start: 2018.4166666666667, 'Years total': 0.08333333333333333, Type: 'Project', Color: '#004000C0' }]
  };

  /** The UI. */
  private readonly mockUi: Indexable =
    { 'CV Generator': { text: 'Lorem ipsum ' }, Search: { text: 'Lorem ' }, Delete: { text: 'L' }, 'Clear Search': { text: 'Lorem ipsum ' }, 'Start All Over': { text: 'Lorem ipsum do' }, Decorations: { text: 'Lorem ipsum' }, Mode: { text: 'Lore' }, 'tag cloud': { text: 'Lorem ips' }, chart: { text: 'Lorem' }, both: { text: 'Lore' }, 'team members': { text: 'Lorem ipsum ' }, Disclaimer: { text: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse c' }, Copyright: { text: 'Lorem ' }, 'Author name': { text: 'Lorem ipsum do' }, 'All rights reserved': { text: 'Lorem ipsum dolor s' }, By: { text: 'Lo' }, Trans: { text: 'Lorem' }, Ed: { text: 'Lo' }, Expires: { text: 'Lorem i' }, 'Link to this heading': { text: 'Lorem ipsum dolor si' }, 'Collapse this heading': { text: 'Lorem ipsum dolor sit' }, 'Expand this heading': { text: 'Lorem ipsum dolor s' }, 'Search for this': { text: 'Lorem ipsum dol' }, 'Go to this project': { text: 'Lorem ipsum dolor ' }, 'Go to top': { text: 'Lorem ips' }, Up: { text: 'Lo' }, 'GitHub version': { text: 'Lorem ipsum do' }, 'Package version': { text: 'Lorem ipsum dol' }, 'Build version': { text: 'Lorem ipsum d' }, 'NPM version': { text: 'Lorem ipsum' }, 'Build status': { text: 'Lorem ipsum ' }, 'Node version': { text: 'Lorem ipsum ' }, 'Last GitHub commit': { text: 'Lorem ipsum dolor ' }, Heroku: { text: 'Lorem ' }, 'Number of languages': { text: 'Lorem ipsum dolor s' }, 'Top language': { text: 'Lorem ipsum ' }, 'Travis build status': { text: 'Lorem ipsum dolor s' }, 'AppVeyor build status': { text: 'Lorem ipsum dolor sit' }, 'Test status': { text: 'Lorem ipsum' }, 'Codecov code coverage status': { text: 'Lorem ipsum dolor sit amet, ' }, 'Coveralls code coverage status': { text: 'Lorem ipsum dolor sit amet, co' }, 'Code coverage list': { text: 'Lorem ipsum dolor ' }, 'npm dependencies': { text: 'Lorem ipsum dolo' }, 'npm devDependencies': { text: 'Lorem ipsum dolor s' }, 'Progressive Web App': { text: 'Lorem ipsum dolor s' }, Sitemap: { text: 'Lorem i' }, 'Third-party licenses': { text: 'Lorem ipsum dolor si' }, Code: { text: 'Lore' }, 'Documentation status': { text: 'Lorem ipsum dolor si' }, Documentation: { text: 'Lorem ipsum d' }, 'licensed under': { text: 'Lorem ipsum do' }, 'MIT license': { text: 'Lorem ipsum' }, 'CC BY 4.0': { text: 'Lorem ips' }, 'Made in Bulgaria': { text: 'Lorem ipsum dolo' }, Bulgaria: { text: 'Lorem ip' }, 'Rose Valley': { text: 'Lorem ipsum' }, 'Project site': { text: 'Lorem ipsum ' }, AI: { text: 'Lo' }, 'Toggle navigation': { text: 'Lorem ipsum dolor' }, 'Certificate image': { text: 'Lorem ipsum dolor' }, Accomplishment: { text: 'Lorem ipsum do' }, Authority: { text: 'Lorem ips' }, Course: { text: 'Lorem ' }, Certificate: { text: 'Lorem ipsum' }, Education: { text: 'Lorem ips' }, Experience: { text: 'Lorem ipsu' }, Project: { text: 'Lorem i' }, Client: { text: 'Lorem ' }, 'Project URL': { text: 'Lorem ipsum' }, Publication: { text: 'Lorem ipsum' }, 'Publication image': { text: 'Lorem ipsum dolor' } };

  /**
   * Constructs the data service.
   * ~constructor
   *
   * @param imageDataService The data service injected dependency.
   * @param httpClient The http client for requests to the server.
   */
  constructor(
    public imageDataService: ImageDataService,
    protected httpClient: HttpClient) {
    super(imageDataService, httpClient);
  }

  /**
   * Retrieves a CV.
   *
   * @returns The CV.
   */
  public getCv(): Observable<Indexable> {
    // tslint:disable-next-line: max-line-length
    const cv = of({ 'Personal data': [{ Caption: 'Lore', 'Personal data': 'Lorem ipsum do', Hidden: 'Lorem', Highlight: 'Lorem ipsum dol' }, { Caption: 'Lorem ipsum d', 'Personal data': 'Lorem ipsum dolo', Hidden: 'Lorem' }, { Caption: 'Lorem ip', 'Personal data': 'Lorem ipsum dol', Hidden: 'Lore' }, { Caption: 'Lorem ipsum dol', 'Personal data': 'Lorem ipsum dolor', Hidden: 'Lore' }, { Caption: 'Lorem ipsum do', 'Personal data': 'Lorem ipsum dolor', Hidden: 'Lorem' }, { Caption: 'Lorem ipsum', 'Personal data': 'Lorem ips', Hidden: 'Lorem' }, { Caption: 'Lorem i', 'Personal data': '', Hidden: 'Lorem' }, { Caption: 'Lorem ipsum dolo', 'Personal data': '', Hidden: 'Lorem' }, { Caption: 'Lorem i', 'Personal data': 'Lorem ipsum dolor sit amet, consectetur a', Hidden: 'Lorem' }, { Caption: 'Lorem', 'Personal data': 'Lorem ipsum dolo', Hidden: 'Lorem' }, { Caption: 'Lorem ', 'Personal data': 'Lorem ipsum dol', Hidden: 'Lorem' }, { Caption: 'Lorem ip', 'Personal data': 'Lorem ipsum dolor sit amet, consectetur a', Hidden: 'Lore' }, { Caption: 'Lorem i', 'Personal data': 'Lorem ipsum dolor sit ame', Hidden: 'Lore' }, { Caption: 'Lorem ip', 'Personal data': 'Lorem ipsum dolor sit amet, con', Hidden: 'Lore' }, { Caption: 'Lorem ips', 'Personal data': 'Lorem ipsum dolor sit amet, conse', Hidden: 'Lore' }, { Caption: 'Lor', 'Personal data': 'Lorem ipsu', Hidden: 'Lore' }, { Caption: 'Lorem ipsum ', 'Personal data': 'Lorem ipsum dolor sit ', Hidden: 'Lore' }], 'Professional experience': [{ Id: 1, From: 33181, To: 43932, Experience: 'Lorem ipsum dolor sit amet, conse', Position: 'Lorem ipsum dolo', Description: ['Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis n', '', 'Lorem ipsum do', '', 'Lorem ipsum dolor sit a', 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusm'], Link: 'Lorem ipsum dolor s', Image: 'Lorem ipsum dolor sit ame', Color: '#FF800080', Highlight: 'Lorem ipsum dol' }, { Id: 2, From: 31376, To: 30335, Experience: 'Lorem ipsum dolor sit amet, consect', Position: 'Lorem ipsum dolor sit amet, consecte', Description: [], Link: '', Image: '', Color: '#FF800080' }], Education: [{ Id: 1, From: 35710, To: 34961, School: 'Lorem ipsum dolor sit amet, con', Field: 'Lorem ipsum d', Major: 'Lorem i', Degree: '', Honors: 'Lorem ipsum dolor sit ', Description: ['Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed d', 'Lorem ipsum dolor sit amet, cons'], Link: 'Lorem ipsum dolor sit amet', Image: 'Lorem ipsum dolor sit amet', Color: '#008080C0', Highlight: '', Credits: 82, Grade: '', Examinations: '', Final: '', GradePct: '', ExaminationsPct: '', FinalPct: '', GradeBG: '', ExaminationsBG: '', FinalBG: '', GradeUS: '', ExaminationsUS: '', FinalUS: '', GradeUK: '', ExaminationsUK: '', FinalUK: '' }, { Id: 2, From: 37444, To: 40520, School: 'Lorem ipsum dolor sit amet, c', Field: 'Lorem ipsum do', Major: 'Lorem ipsum dolor sit amet, consec', Degree: 'Lorem ipsum dolor sit amet', Honors: '', Description: ['Lorem ipsum dolor sit amet, consectetur '], Link: 'Lorem ipsum dolor', Image: 'Lorem ipsum dolor sit amet', Color: '#008080C0', Highlight: '', Credits: '', Grade: '', Examinations: '', Final: '', GradePct: '', ExaminationsPct: '', FinalPct: '', GradeBG: '', ExaminationsBG: '', FinalBG: '', GradeUS: '', ExaminationsUS: '', FinalUS: '', GradeUK: '', ExaminationsUK: '', FinalUK: '' }, { Id: 3, From: 30449, To: 31706, School: 'Lorem ipsum dolor sit amet, consectet', Field: 'Lorem ipsum dolo', Major: 'Lorem ipsum', Degree: 'Lorem', Honors: 'Lorem ipsum dolor s', Description: ['Lorem ipsum dolor sit amet, consect', 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, s'], Link: 'Lorem ipsum dolor sit amet, ', Image: 'Lorem ipsum dolor sit amet, consectetur a', Color: '#008080C0', Highlight: 'Lorem ipsum dol', Credits: '', Grade: 6, Examinations: 6, Final: 6, GradePct: 1, ExaminationsPct: 1, FinalPct: 1, GradeBG: 'L', ExaminationsBG: 'L', FinalBG: 'L', GradeUS: 'Lor', ExaminationsUS: 'Lor', FinalUS: 'Lore', GradeUK: 'Lorem', ExaminationsUK: 'Lorem i', FinalUK: 'Lorem' }, { Id: 4, From: 33926, To: 30615, School: 'Lorem ipsum dolor sit amet, consectetur adi', Field: '', Major: 'Lorem ipsum dolor sit am', Degree: '', Honors: 'Lorem ipsum dolo', Description: ['Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ei', 'Lorem ipsum dolor sit amet, consectetur adip'], Link: 'Lorem ipsum dolor sit amet, ', Image: 'Lorem ipsum dolor', Color: '#008080C0', Highlight: '', Credits: '', Grade: 6, Examinations: '', Final: '', GradePct: 1, ExaminationsPct: '', FinalPct: '', GradeBG: 'L', ExaminationsBG: '', FinalBG: '', GradeUS: 'Lore', ExaminationsUS: '', FinalUS: '', GradeUK: 'Lorem', ExaminationsUK: '', FinalUK: '' }, { Id: 5, From: 28932, To: 31633, School: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Field: '', Major: '', Degree: '', Honors: 'Lorem ipsum dolo', Description: [], Link: 'Lorem ipsum dolor sit ', Image: 'Lorem ipsum', Color: '#008080C0', Highlight: '', Credits: '', Grade: 6, Examinations: '', Final: '', GradePct: 1, ExaminationsPct: '', FinalPct: '', GradeBG: 'L', ExaminationsBG: '', FinalBG: '', GradeUS: 'Lore', ExaminationsUS: '', FinalUS: '', GradeUK: 'Lorem', ExaminationsUK: '', FinalUK: '' }], Certifications: [{ Id: 1, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', 'Authority name': 'Lorem ipsum', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dol', Type: 'Lorem ipsum d', Level: 'Lorem ipsu', Location: 'Lorem ipsum', Started: 39964, Completed: 41323, Expiration: '', 'Certificate number': 'Lorem ipsum dolor', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing el', 'Certificate image': 'Lorem ipsum dolor sit amet, c', 'Certificate image URL': '', 'Certificate logo': 'Lorem ipsum dolor sit amet, c', 'Certificate tag': '', Color: '#7F00FFC0' }, { Id: 2, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing ', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod temp', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem ipsum d', Level: 'Lorem ips', Location: 'Lorem ipsum dolor si', Started: 44300, Completed: 39767, Expiration: 39725, 'Certificate number': 'Lorem ipsum dolo', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididun', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', 'Certificate image URL': '', 'Certificate logo': 'Lorem ipsum dolor sit ', 'Certificate tag': 'Lorem ipsum dolor sit', Color: '#7F00FFC0' }], Languages: [{ Language: 'Lorem ips', Level: 'Lorem ipsum dolor sit amet, con', Score: 5, Share: 35 }, { Language: 'Lorem i', Level: 'Lorem ipsum dolor sit amet, c', Score: 4, Share: 27 }, { Language: 'Lorem i', Level: 'Lorem ipsum dolor sit amet,', Score: 2, Share: 9 }, { Language: 'Lorem i', Level: 'Lorem ipsum dolor sit ', Score: 1, Share: 9 }, { Language: 'Lorem ', Level: 'Lorem ipsum dolor sit ', Score: 1, Share: 4 }, { Language: 'Lorem ', Level: 'Lorem ipsum dolor sit ', Score: 1, Share: 4 }, { Language: 'Lorem i', Level: 'Lorem ipsum dolor sit ', Score: 1, Share: 4 }, { Language: 'Lorem i', Level: 'Lorem ipsum dolor sit ', Score: 1, Share: 4 }], Courses: [{ Id: 3, Name: 'Lorem ipsum do', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do', 'Authority name': 'Lorem ipsu', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum do', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 40508, Completed: 39071, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 4, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor ', 'Authority name': 'Lorem', 'Authority URL': 'Lorem ipsum dolor sit', 'Authority image': 'Lorem ips', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 43518, Completed: 44428, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 5, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempo', 'Authority name': 'Lorem i', 'Authority URL': 'Lorem ipsum dolor sit a', 'Authority image': 'Lorem ipsum', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 42546, Completed: 47336, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 6, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit,', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor in', 'Authority name': 'Lorem ipsum do', 'Authority URL': 'Lorem ipsum dolor sit amet, con', 'Authority image': 'Lorem ipsum dolor ', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 42503, Completed: 46293, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 7, Name: 'Lorem ipsum dolor sit amet, consecte', URL: 'Lorem ipsum dolor sit amet, consectetur', 'Authority name': 'Lorem', 'Authority URL': 'Lorem ipsum dolor sit', 'Authority image': 'Lorem ips', Type: 'Lorem ', Level: 'Lorem ipsu', Location: 'Lorem', Started: 45414, Completed: 43820, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 8, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', URL: 'Lorem ipsum dolor sit amet, consectetur ', 'Authority name': 'Lorem ipsum d', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 41444, Completed: 44911, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 9, Name: 'Lorem ipsum dolor sit amet, consectet', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, s', 'Authority name': 'Lorem', 'Authority URL': 'Lorem ipsum dolor sit', 'Authority image': 'Lorem ips', Type: 'Lorem ', Level: 'Lorem ipsu', Location: 'Lorem', Started: 41207, Completed: 41766, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 10, Name: 'Lorem ipsum dolor s', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem ipsu', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor si', Started: 39106, Completed: 39675, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing ', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 11, Name: 'Lorem ipsum dolor sit amet, consectet', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem', Level: 'Lorem ipsum', Location: 'Lorem ipsum dolor si', Started: 45667, Completed: 43015, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmo', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 12, Name: 'Lorem ipsum dolor sit amet, con', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem', Level: 'Lorem ipsum', Location: 'Lorem ipsum dolor si', Started: 43413, Completed: 42903, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 13, Name: 'Lorem ipsum dolor sit amet, consectetur ', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Authority name': 'Lorem ipsum ', 'Authority URL': 'Lorem ipsum dolor si', 'Authority image': 'Lorem ipsum dolo', Type: 'Lorem ', Level: 'Lorem ipsu', Location: 'Lorem', Started: 39934, Completed: 43901, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 14, Name: 'Lorem ipsum dolor sit amet, consectetur ', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing e', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem ipsum', Level: 'Lorem ip', Location: 'Lorem ipsum dolor si', Started: 44088, Completed: 44288, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod ', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 15, Name: 'Lorem ipsum dolor sit amet, consectetur adipisc', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing e', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem ipsum', Level: 'Lorem ip', Location: 'Lorem ipsum dolor si', Started: 39095, Completed: 40600, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor ', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 16, Name: 'Lorem ipsum dolor sit amet, consecte', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing e', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem ipsum', Level: 'Lorem ip', Location: 'Lorem ipsum dolor si', Started: 45488, Completed: 45316, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eius', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 17, Name: 'Lorem ipsum dolor sit amet, consect', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', 'Authority name': 'Lorem ipsum dolor s', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum dolor sit a', Type: 'Lorem ipsu', Level: 'Lorem ip', Location: 'Lorem ipsum dolor si', Started: 43204, Completed: 40099, Expiration: '', 'Certificate number': 'Lorem ipsum dolor sit a', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labor', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiu', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 18, Name: 'Lorem ipsum dolor sit ', URL: 'Lorem ipsum dolor sit amet, consectetur adip', 'Authority name': 'Lorem', 'Authority URL': 'Lorem ipsum dolor sit', 'Authority image': 'Lorem ips', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 43623, Completed: 42807, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 19, Name: 'Lorem ipsum dolor sit amet, consectetur adipi', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incid', 'Authority name': 'Lorem ipsum d', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ', Level: 'Lorem ipsum ', Location: 'Lorem', Started: 46592, Completed: 41932, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 20, Name: 'Lorem ipsum do', URL: 'Lorem ipsum dolor sit amet, consecte', 'Authority name': 'Lorem ipsum d', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 44122, Completed: 43180, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 21, Name: 'Lorem ipsum ', URL: 'Lorem ipsum dolor sit amet, consec', 'Authority name': 'Lorem ipsum d', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ', Level: 'Lorem ip', Location: 'Lorem', Started: 44143, Completed: 47296, Expiration: '', 'Certificate number': 'Lorem ipsum', 'Certificate URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image': 'Lorem ipsum dol', 'Certificate image URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 22, Name: 'Lorem ipsum dolo', URL: 'Lorem ipsum dolor sit ame', 'Authority name': 'Lorem ip', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum ', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem', Started: 43707, Completed: 41417, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 23, Name: 'Lorem ipsu', URL: '', 'Authority name': '', 'Authority URL': '', 'Authority image': '', Type: 'Lorem ipsu', Level: '', Location: 'Lorem ipsum dolo', Started: 41121, Completed: 43392, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#FF800080' }, { Id: 24, Name: 'Lorem ipsum dolor sit amet, ', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet,', 'Authority URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Authority image': 'Lorem ipsum dol', Type: 'Lorem ipsum dolor s', Level: '', Location: 'Lorem ipsum dol', Started: 42112, Completed: 36508, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 25, Name: 'Lorem ipsum dolor sit amet, c', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet,', 'Authority URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Authority image': 'Lorem ipsum dol', Type: 'Lorem ipsum dolor s', Level: '', Location: 'Lorem ipsum dol', Started: 38525, Completed: 38545, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 26, Name: 'Lorem ipsum dolor sit amet, con', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet,', 'Authority URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Authority image': 'Lorem ipsum dol', Type: 'Lorem ipsum dolor s', Level: '', Location: 'Lorem ipsum dolor ', Started: 34893, Completed: 35776, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 27, Name: 'Lorem ipsum do', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do e', 'Authority name': 'Lorem ipsum dolor sit amet, co', 'Authority URL': 'Lorem ipsum dolor sit amet, consectetur adipisc', 'Authority image': 'Lorem ipsu', Type: 'Lorem ipsum ', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor sit amet, consectetu', Started: 35545, Completed: 35839, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 28, Name: 'Lorem ipsum dolor sit amet, consectetur adipisci', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ei', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 37011, Completed: 37140, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 29, Name: 'Lorem ipsum dolor sit amet, consectetu', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ei', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 39399, Completed: 38119, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 30, Name: 'Lorem ipsum dolor sit amet, consectetur', URL: '', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsum ', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor si', Started: 34782, Completed: 33149, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 31, Name: 'Lorem ipsum dolor sit amet, co', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do e', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsum ', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 36331, Completed: 36973, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 32, Name: 'Lorem ipsum dolor sit amet, ', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do e', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 35663, Completed: 39020, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 33, Name: 'Lorem ipsum do', URL: '', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 37474, Completed: 36075, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 34, Name: 'Lorem ipsum dolor si', URL: '', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsu', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 35423, Completed: 37280, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 35, Name: 'Lorem ipsum dolor sit amet, conse', URL: '', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor sit amet', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsu', Level: 'Lorem ip', Location: 'Lorem ipsum dolor ', Started: 33736, Completed: 34691, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consectetur', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 36, Name: 'Lorem ipsum dolor sit amet, consectetur adipisci', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore ma', 'Authority name': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Authority URL': 'Lorem ipsum dolor sit amet, c', 'Authority image': 'Lorem ip', Type: 'Lorem i', Level: '', Location: 'Lorem ipsum dolor', Started: 34483, Completed: 36853, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 37, Name: 'Lorem ipsum dol', URL: '', 'Authority name': 'Lorem ipsum dolor sit ', 'Authority URL': 'Lorem ipsum dolor', 'Authority image': 'Lorem ipsum dolor sit amet', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolo', Started: 33417, Completed: 34716, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 38, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inc', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veni', 'Authority name': 'Lorem ipsum dolor sit amet, consectetur adipiscin', 'Authority URL': 'Lorem ipsum dolor ', 'Authority image': 'Lorem ip', Type: 'Lorem ipsum d', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolo', Started: 32340, Completed: 34670, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': 'Lorem ipsum dolor sit amet, consecte', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 39, Name: 'Lorem ipsum dol', URL: '', 'Authority name': 'Lorem ipsum do', 'Authority URL': 'Lorem ipsum dolor', 'Authority image': 'Lorem ipsum dolor ', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolo', Started: 32683, Completed: 35854, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 40, Name: 'Lorem ipsum dolor sit amet, ', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consectetur', 'Authority URL': 'Lorem ipsum dolor sit amet, ', 'Authority image': 'Lorem ipsum dolor sit amet, consectetur a', Type: 'Lorem ipsum dol', Level: 'Lorem ip', Location: 'Lorem ipsum dolo', Started: 34802, Completed: 34233, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 41, Name: 'Lorem ipsum dolor sit amet, ', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consectetur', 'Authority URL': 'Lorem ipsum dolor sit amet, ', 'Authority image': 'Lorem ipsum dolor sit amet, consectetur a', Type: 'Lorem ipsum dol', Level: 'Lorem ip', Location: 'Lorem ipsum dolo', Started: 33532, Completed: 35310, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 42, Name: 'Lorem ipsum dolor sit amet, cons', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consecte', 'Authority URL': 'Lorem ipsum dolor sit amet, ', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem i', Level: '', Location: 'Lorem ipsum dolor sit amet, conse', Started: 32148, Completed: 29245, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 43, Name: 'Lorem ipsum dolor sit ', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consecte', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum', Type: 'Lorem i', Level: '', Location: 'Lorem ipsum dolor sit amet,', Started: 30749, Completed: 32040, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 44, Name: 'Lorem ipsum dolor sit ', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consecte', 'Authority URL': 'Lorem ipsum dolor sit amet, ', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 30919, Completed: 33210, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 45, Name: 'Lorem ipsum dolor sit', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consecte', 'Authority URL': 'Lorem ipsum dolor sit amet, ', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ipsum dol', Level: 'Lorem ipsum ', Location: 'Lorem ipsum dolor ', Started: 33222, Completed: 31505, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 46, Name: 'Lorem ipsum dolor sit ', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consecte', 'Authority URL': 'Lorem ipsum dolor sit amet, ', 'Authority image': 'Lorem ipsum dolor', Type: 'Lorem ipsum dol', Level: 'Lorem ip', Location: 'Lorem ipsum dolor ', Started: 31208, Completed: 30261, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 47, Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore ', URL: '', 'Authority name': 'Lorem ipsum ', 'Authority URL': 'Lorem ipsum dolor sit am', 'Authority image': 'Lorem ipsum', Type: 'Lore', Level: '', Location: 'Lorem ipsum dolor ', Started: 27893, Completed: 32882, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 48, Name: 'Lorem ipsum do', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet,', 'Authority URL': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Authority image': 'Lorem ipsum dol', Type: 'Lore', Level: '', Location: 'Lorem ipsum dolor ', Started: 32165, Completed: 33909, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 49, Name: 'Lorem ipsum dolo', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consectetur ad', 'Authority URL': 'Lorem ipsum dolor sit ', 'Authority image': 'Lorem ipsum', Type: 'Lorem ipsum dol', Level: 'Lorem ip', Location: 'Lorem ipsum dolor ', Started: 26877, Completed: 29444, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }, { Id: 50, Name: 'Lorem ipsum d', URL: '', 'Authority name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod', 'Authority URL': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate v', 'Authority image': 'Lorem ipsum dolo', Type: 'Lorem ipsum d', Level: '', Location: 'Lorem ipsum dolor ', Started: 29811, Completed: 28275, Expiration: '', 'Certificate number': '', 'Certificate URL': '', 'Certificate image': '', 'Certificate image URL': '', 'Certificate logo': '', 'Certificate tag': '', Color: '#C000C080' }], Publications: [{ Id: 1, Article: 'Lorem ipsum dolor sit amet', 'Article author': 'Lorem ipsum dolor sit amet', 'Article date': '', Title: 'Lorem ipsum dolor si', Subtitle: 'Lorem ipsum dolor sit a', 'Translation Article': 'Lorem ipsum dolor sit', 'Translation Title': 'Lorem ipsum dolor s', 'Translation Subtitle': 'Lorem ipsum dolor sit ame', Translator: 'Lorem ipsum do', Editor: 'Lorem ipsum dolor s', Publisher: 'Lorem', 'Publication date': 'Lore', Type: 'Lorem ips', Author: 'Lorem ipsum dolor sit amet, consectetur adi', City: 'Lorem', 'Page count': 'Lorem', Pages: 'Lorem ips', Size: 'Lorem ipsu', Format: 'Lorem ips', ISBN: 'Lorem ipsum d', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco lab', 'Publication image': 'Lorem ipsum dolor sit am', Description: ['Lorem ipsum dolor sit ', 'Lorem ipsum do', 'Lore', 'Lorem ipsum dolor si', 'Lorem ipsum dolor sit a', 'Lorem i', 'Lorem', 'Lore', 'Lore', 'Lorem', 'Lorem ipsum ', 'Lorem ipsum dolor ', ['Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco lab', 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate ve']] }, { Id: 2, Article: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et d', 'Article author': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Article date': '', Title: 'Lorem ipsum dol', Subtitle: 'Lorem ipsum dolor sit a', 'Translation Article': '', 'Translation Title': 'Lorem ipsum dolor', 'Translation Subtitle': 'Lorem ipsum dolor ', Translator: 'Lorem ipsum do', Editor: 'Lorem ipsum ', Publisher: 'Lorem', 'Publication date': 'Lore', Type: 'Lorem ips', Author: 'Lorem ipsum dolor sit amet, consectetur ', City: 'Lorem', 'Page count': 'Lorem', Pages: '', Size: 'Lorem ipsu', Format: 'Lorem ips', ISBN: 'Lorem ipsum d', URL: 'Lorem ipsum dolor sit amet, consectetur adipiscing', 'Publication image': 'Lorem ipsum dolor s', Description: ['Lorem ipsum dolor sit amet, consectetu', 'Lorem ipsum dolor', 'Lorem ipsum dolor sit', 'Lorem', 'Lore', 'Lore', 'Lorem', 'Lorem', 'Lorem ipsum dolor '] }], 'Countries visited': ['Lorem ', 'Lorem i', 'Lorem i', 'Lorem i', 'Lorem ip', 'Lorem i', 'Lorem i', 'Lorem ', 'Lorem ', 'Lorem ipsum', 'Lo', 'Lorem ', 'Lorem', 'Lorem ', 'Lorem i', 'Lorem ', 'Lorem ', 'Lorem ips', 'Lorem i', 'Lorem ipsum', 'Lorem i', 'Lorem ipsum do', 'Lorem', 'Lorem '] });

    return cv;
  }

  /**
   * Retrieves a projects array.
   *
   * @returns The projects array.
   */
  public getProjects(): Observable<Indexable> {
    // tslint:disable-next-line: max-line-length
    const projects = of([{ Id: 1, 'Project name': 'Lorem ipsum dolor sit ', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod t', Client: 'Lorem ip', Logo: 'Lorem ipsu', Country: '', Industry: 'Lorem ipsu', 'Project type': 'Lorem', 'System type': 'Lorem ipsum do', Platform: 'Lorem ipsum dolor sit amet, c', Architecture: 'Lorem ipsum dolo', 'Languages and notations': 'Lorem ipsu', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis ', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod t', Role: 'Lorem', Position: 'Lorem ips', From: 46984, To: 41514, 'Months total': 1, 'Duration total': 'Lorem i', Reference: '', Links: '', Photos: 'Lorem ipsum d', Attribution: 'Lorem ipsum do', 'Client link': '', Period: 'Lorem ipsu', Color: '#004000C0' }, { Id: 2, 'Project name': 'Lorem ipsum dolor sit am', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Client: 'Lorem ip', Logo: 'Lorem ipsu', Country: '', Industry: 'Lorem ipsu', 'Project type': 'Lorem', 'System type': 'Lor', Platform: 'Lorem ipsum dolor sit amet, consectetu', Architecture: 'Lorem ipsum d', 'Languages and notations': 'Lorem ips', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing el', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod t', Role: 'Lorem', Position: 'Lorem ips', From: 43383, To: 39929, 'Months total': 1, 'Duration total': 'Lorem i', Reference: '', Links: 'Lorem ipsum dolor sit amet, consectetur adipisc', Photos: 'Lorem ipsum dol', Attribution: 'Lorem ipsum do', 'Client link': '', Period: 'Lorem ipsu', Color: '#004000C0' }, { Id: 3, 'Project name': 'Lorem ipsum dolor sit amet', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', Client: 'Lorem ip', Logo: 'Lorem ipsu', Country: '', Industry: 'Lorem ipsu', 'Project type': 'Lorem', 'System type': 'Lor', Platform: 'Lorem ipsum dolor', Architecture: 'Lorem ipsum d', 'Languages and notations': 'Lorem ipsum dolor sit amet', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor i', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod t', Role: 'Lorem', Position: 'Lorem ips', From: 44512, To: 42113, 'Months total': 1, 'Duration total': 'Lorem i', Reference: '', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Photos: 'Lorem ipsum dolor', Attribution: 'Lorem ipsum do', 'Client link': '', Period: 'Lorem ipsu', Color: '#004000C0' }, { Id: 4, 'Project name': 'Lorem ipsum dolor sit am', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tem', Client: 'Lorem ip', Logo: 'Lorem ipsu', Country: '', Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsu', 'System type': 'Lorem ip', Platform: 'Lorem ipsum dolor sit amet, ', Architecture: 'Lorem ipsum dolor sit amet, consectet', 'Languages and notations': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incid', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepte', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inc', Role: 'Lorem', Position: 'Lorem ips', From: 45278, To: 44268, 'Months total': 3, 'Duration total': 'Lorem ip', Reference: '', Links: 'Lorem ipsum dolor sit amet, consectet', Photos: 'Lorem ipsum dolo', Attribution: 'Lorem ipsum do', 'Client link': '', Period: 'Lorem ipsu', Color: '#004000C0' }, { Id: 5, 'Project name': 'Lorem ipsum dolor sit amet, c', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, s', Client: 'Lorem ip', Logo: 'Lorem ipsu', Country: '', Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum dolo', 'System type': 'Lorem i', Platform: 'Lorem ipsum dolo', Architecture: 'Lorem ipsu', 'Languages and notations': '', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod t', Role: 'Lorem', Position: 'Lorem ips', From: 44255, To: 44201, 'Months total': 1, 'Duration total': 'Lorem i', Reference: '', Links: '', Photos: 'Lorem ipsum dolor sit', Attribution: 'Lorem ipsum do', 'Client link': '', Period: 'Lorem ipsu', Color: '#004000C0' }, { Id: 6, 'Project name': 'Lorem ipsum dolor sit amet, consecte', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusm', Client: 'Lorem ipsum dolor sit', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem ipsu', 'Project type': 'Lor', 'System type': 'Lorem i', Platform: 'Lorem ipsum dolor sit amet, consec', Architecture: 'Lorem ipsum dolor sit amet, consectetur adipi', 'Languages and notations': 'Lorem ipsum d', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inc', 'Team size': 1, Responsibilities: 'Lorem ipsum', Role: 'Lorem ipsum dolo', Position: 'Lorem ipsum dolo', From: 44700, To: 42259, 'Months total': 9, 'Duration total': 'Lorem ip', Reference: 'Lorem', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut la', Photos: 'Lorem ipsum do', Attribution: 'Lorem ipsu', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 7, 'Project name': 'Lorem ipsum dolor sit', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commo', Client: 'Lorem ipsum dolor sit amet, c', Logo: 'Lorem ipsum dolor sit', Country: 'Lorem ', Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum d', 'System type': 'Lorem i', Platform: 'Lorem ipsum do', Architecture: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Languages and notations': 'Lorem ipsum do', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing ', 'Team size': 1, Responsibilities: 'Lorem ipsum', Role: 'Lorem ipsum dolo', Position: 'Lorem ipsum dolo', From: 38581, To: 40599, 'Months total': 8, 'Duration total': 'Lorem ip', Reference: 'Lorem ip', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore', Photos: 'Lorem ips', Attribution: 'Lorem ipsum dolor sit amet, co', 'Client link': 'Lorem ipsum dolor sit amet, consectetur adipisc', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 8, 'Project name': 'Lorem ipsum dolor sit amet, cons', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in volupt', Client: 'Lorem ipsum dolor sit amet, co', Logo: 'Lorem ipsum dolor sit ', Country: 'Lorem ', Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum', 'System type': 'Lorem i', Platform: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed', Architecture: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt u', 'Languages and notations': 'Lorem ipsum dolor si', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Role: 'Lorem ipsum do', Position: 'Lorem ipsum dolo', From: 41540, To: 43726, 'Months total': 11, 'Duration total': 'Lorem ips', Reference: 'Lore', Links: 'Lorem ipsum dolor sit amet, ', Photos: 'Lorem ipsum dol', Attribution: 'Lorem ipsum dol', 'Client link': 'Lorem ipsum dolor sit amet, c', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 9, 'Project name': 'Lorem ipsum dolor sit amet, consectetur ', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea ', Client: 'Lorem ipsum dolor sit amet, co', Logo: 'Lorem ipsum dolor sit ', Country: 'Lorem ', Industry: 'Lorem ipsum dolor sit amet, co', 'Project type': 'Lorem ips', 'System type': 'Lorem ipsum dol', Platform: 'Lorem ipsum dolor sit amet,', Architecture: 'Lorem ipsum dolor si', 'Languages and notations': 'Lorem ips', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Role: 'Lorem ipsum do', Position: 'Lorem ipsum dolo', From: 41653, To: 39567, 'Months total': 89, 'Duration total': 'Lorem ip', Reference: 'Lorem', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut la', Photos: 'Lorem ipsum dolor sit ', Attribution: 'Lorem ipsum dolor sit amet, c', 'Client link': 'Lorem ipsum dolor sit amet, c', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 10, 'Project name': 'Lorem i', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore', Client: 'Lorem ipsum dolor sit amet, co', Logo: 'Lorem ipsum dolor sit ', Country: 'Lorem ', Industry: 'Lorem ipsum dolor sit amet, co', 'Project type': 'Lorem ips', 'System type': 'Lorem ipsum dolor', Platform: 'Lorem ipsum dolor sit amet,', Architecture: 'Lorem ipsum dolor si', 'Languages and notations': 'Lorem ipsum dol', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Role: 'Lorem ipsum do', Position: 'Lorem ipsum dolo', From: 40430, To: 41940, 'Months total': 10, 'Duration total': 'Lorem ips', Reference: 'Lorem', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore e', Photos: 'Lorem ipsum', Attribution: 'Lorem ipsum dolor sit amet,', 'Client link': 'Lorem ipsum dolor sit amet, c', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 11, 'Project name': 'Lorem ipsum dolor sit amet, cons', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing e', Client: 'Lorem ipsum dolor sit amet, co', Logo: 'Lorem ipsum dolor sit ', Country: 'Lorem ', Industry: 'Lorem ip', 'Project type': 'Lorem ips', 'System type': 'Lorem i', Platform: 'Lorem ipsum do', Architecture: 'Lorem ipsum d', 'Languages and notations': 'Lo', 'IDEs and Tools': 'Lorem ipsum dolor sit a', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur', Role: 'Lorem ipsum do', Position: 'Lorem ipsum dolo', From: 40783, To: 39935, 'Months total': 5, 'Duration total': 'Lorem ip', Reference: 'Lorem ', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad ', Photos: 'Lorem ipsum', Attribution: 'Lorem ipsum dolor sit amet, c', 'Client link': 'Lorem ipsum dolor sit amet, c', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 12, 'Project name': 'Lorem ipsum ', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut ', Client: 'Lorem ipsum dolor sit amet, co', Logo: 'Lorem ipsum dolor sit ', Country: 'Lorem ', Industry: 'Lorem ipsum dolor s', 'Project type': 'Lorem ipsum dolor sit amet, consectetur adipisc', 'System type': 'Lorem ipsum dolor', Platform: 'Lorem ipsum dolor s', Architecture: 'Lorem i', 'Languages and notations': 'Lorem ipsum ', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Role: 'Lorem ipsum do', Position: 'Lorem ips', From: 41449, To: 40185, 'Months total': 18, 'Duration total': 'Lorem ipsum dol', Reference: 'Lorem ipsum dolor sit amet, conse', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et', Photos: 'Lorem ipsum dolo', Attribution: 'Lorem ipsum dolor sit amet, co', 'Client link': 'Lorem ipsum dolor sit amet, c', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 13, 'Project name': 'Lorem ips', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud e', Client: 'Lorem ipsum dolor sit amet, co', Logo: 'Lorem ipsum dolor sit ', Country: 'Lorem ', Industry: 'Lorem ipsum dolor sit amet, co', 'Project type': 'Lorem ips', 'System type': 'Lorem ipsum dolor', Platform: 'Lorem ipsum dolor sit amet, ', Architecture: 'Lorem ipsum', 'Languages and notations': 'Lorem', 'IDEs and Tools': 'Lorem ipsum dolor sit ', 'Team size': 3, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, s', Role: 'Lorem ips', Position: 'Lorem ips', From: 38698, To: 36017, 'Months total': 9, 'Duration total': 'Lorem ip', Reference: 'Lor', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut', Photos: 'Lorem ipsum d', Attribution: 'Lorem ipsum dolor sit amet, consectetur ', 'Client link': 'Lorem ipsum dolor sit amet, c', Period: 'Lorem ipsu', Color: '#00C00060' }, { Id: 14, 'Project name': 'Lorem ipsum dolor sit ', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut', Client: 'Lorem ipsum dolor sit amet, consectetur adipisci', Logo: 'Lorem ipsum', Country: 'Lorem i', Industry: 'Lorem ipsum', 'Project type': 'Lor', 'System type': 'Lorem i', Platform: 'Lorem ipsum dolor sit am', Architecture: 'Lorem ipsum dolor sit ', 'Languages and notations': 'Lo', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consecte', 'Team size': 4, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed d', Role: 'Lorem ipsum dol', Position: 'Lorem ips', From: 34794, To: 38931, 'Months total': 8, 'Duration total': 'Lorem ip', Reference: 'Lorem i', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor sit am', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 15, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod', Client: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in volupt', Logo: 'Lorem ipsum dolor sit a', Country: 'Lorem ipsum dolor sit amet, consectetur a', Industry: 'Lorem ipsum dolor si', 'Project type': 'Lor', 'System type': 'Lor', Platform: 'Lorem ipsum dolor', Architecture: 'Lorem ipsum dolor sit amet, consectetur ', 'Languages and notations': 'Lorem ipsum dolor sit amet, cons', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ul', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat null', Role: 'Lorem ipsum dol', Position: 'Lorem ips', From: 35447, To: 35800, 'Months total': 17, 'Duration total': 'Lorem ipsum dol', Reference: 'Lorem', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Photos: 'Lorem ipsu', Attribution: 'Lorem ipsum dolor sit', 'Client link': 'Lorem ipsum dolor sit', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 16, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusm', Client: 'Lorem ipsum dolor', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum ', 'System type': 'Lor', Platform: 'Lorem ipsum dolor sit am', Architecture: 'Lorem i', 'Languages and notations': 'Lorem ipsum dolor sit amet, consectetu', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur ad', 'Team size': 5, Responsibilities: 'Lorem ipsum dolor si', Role: 'Lorem ips', Position: 'Lorem ips', From: 34885, To: 35814, 'Months total': 3, 'Duration total': 'Lorem ip', Reference: 'Lorem ip', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 17, 'Project name': 'Lorem ipsum d', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Client: 'Lorem ipsum dolor sit amet', Logo: 'Lorem ipsum dolor sit amet, co', Country: 'Lorem ip', Industry: 'Lor', 'Project type': 'Lorem ipsum dolor ', 'System type': 'Lorem ipsum ', Platform: 'Lorem ipsum dolor sit am', Architecture: 'Lorem ipsum dolor sit ame', 'Languages and notations': 'Lorem ipsum dolor sit ame', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, ', 'Team size': 2, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore', Role: 'Lorem ips', Position: 'Lorem ips', From: 40737, To: 33903, 'Months total': 3, 'Duration total': 'Lorem ip', Reference: 'Lorem i', Links: 'Lorem ipsum dolor sit amet, consect', Photos: 'Lorem ipsum dolor', Attribution: 'Lorem ipsum dolor sit amet', 'Client link': 'Lorem ipsum dolor sit amet, consect', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 18, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempo', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt', Client: 'Lorem ipsum dolor sit amet, cons', Logo: 'Lorem ipsum dolor sit amet, consecte', Country: 'Lorem ip', Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dolor ', 'System type': 'Lorem i', Platform: 'Lorem ipsum do', Architecture: 'Lorem ipsum dolor sit amet, conse', 'Languages and notations': 'Lo', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing ', 'Team size': 2, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna ', Role: 'Lorem ips', Position: 'Lorem ips', From: 38993, To: 37109, 'Months total': 12, 'Duration total': 'Lorem ips', Reference: 'Lorem', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor sit amet, consectetur adipisc', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 19, 'Project name': 'Lorem', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labo', Client: 'Lorem ipsum dolor sit ame', Logo: 'Lorem ipsum dolor s', Country: 'Lorem i', Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dolo', 'System type': 'Lorem ipsum dolo', Platform: 'Lorem ipsum dolor sit', Architecture: 'Lorem ipsum ', 'Languages and notations': 'Lorem ipsum dolor sit amet, consectetur adipisc', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing', 'Team size': 2, Responsibilities: 'Lorem ipsum dolor sit ame', Role: 'Lorem ipsum dolo', Position: 'Lorem ips', From: 40798, To: 34650, 'Months total': 1, 'Duration total': 'Lorem i', Reference: 'Lorem ipsu', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor sit amet, co', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 20, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adip', Scope: 'Lorem ipsum dolor sit amet, consect', Client: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.', Logo: 'Lorem ipsum dolor sit amet, consectetur ', Country: 'Lorem ip', Industry: 'Lorem ipsum d', 'Project type': 'Lorem ipsum do', 'System type': 'Lorem i', Platform: 'Lorem ipsum dolo', Architecture: 'Lorem ips', 'Languages and notations': 'Lor', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, con', 'Team size': 3, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ', Role: 'Lorem ips', Position: 'Lorem ips', From: 34119, To: 35892, 'Months total': 2, 'Duration total': 'Lorem ip', Reference: 'Lorem', Links: 'Lorem ipsum dolor sit amet, consectetur adi', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor sit amet, consectetur adipi', Period: 'Lorem ipsum', Color: '#000040C0' }, { Id: 21, 'Project name': 'Lorem', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Client: 'Lorem ipsum dolor s', Logo: 'Lorem ipsum dolor sit a', Country: 'Lorem ip', Industry: 'Lor', 'Project type': 'Lorem ipsum dolor ', 'System type': 'Lor', Platform: 'Lorem ipsum dolor sit am', Architecture: 'Lorem ipsum dolor sit ame', 'Languages and notations': 'Lorem ipsum dolor sit amet, consectet', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', 'Team size': 2, Responsibilities: 'Lorem ipsum dolor sit amet, consectetu', Role: 'Lorem ipsum dolor s', Position: 'Lorem ips', From: 36227, To: 36788, 'Months total': 8, 'Duration total': 'Lorem ip', Reference: 'Lorem ', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor sit ame', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 22, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut ', Client: 'Lorem ipsum dol', Logo: 'Lorem ipsum dolor', Country: 'Lorem ip', Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dolor ', 'System type': 'Lorem ipsum ', Platform: 'Lorem ip', Architecture: 'Lorem ipsum dolor s', 'Languages and notations': 'Lorem ipsum ', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut la', Role: 'Lorem ipsum dolo', Position: 'Lorem ips', From: 36655, To: 37027, 'Months total': 4, 'Duration total': 'Lorem ip', Reference: 'Lorem ', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor si', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 23, 'Project name': 'Lorem ipsum dolor sit amet, cons', Scope: 'Lorem ipsum dolor sit amet, cons', Client: 'Lorem ips', Logo: 'Lorem ipsum d', Country: 'Lorem ip', Industry: 'Lorem ip', 'Project type': 'Lor', 'System type': 'Lorem i', Platform: 'Lorem i', Architecture: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do ei', 'Languages and notations': '', 'IDEs and Tools': 'Lorem ipsum dolor sit', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', Role: 'Lorem ipsu', Position: 'Lorem ips', From: 39586, To: 33638, 'Months total': 2, 'Duration total': 'Lorem ip', Reference: 'Lorem i', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor sit amet', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 24, 'Project name': 'Lorem ipsum dol', Scope: 'Lorem ipsum dolor sit amet, consectetur', Client: 'Lorem ipsum dolor sit', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem ipsum dolor ', 'Project type': 'Lorem ipsum dolor ', 'System type': 'Lorem i', Platform: 'Lorem i', Architecture: 'Lorem ipsum d', 'Languages and notations': '', 'IDEs and Tools': '', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit a', Role: 'Lorem ipsum dolo', Position: 'Lorem ips', From: 39047, To: 40186, 'Months total': 1, 'Duration total': 'Lorem i', Reference: 'Lor', Links: '', Photos: 'Lorem ips', Attribution: '', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 25, 'Project name': 'Lorem ip', Scope: 'Lorem ipsum dolor sit ', Client: 'Lorem ipsum dolor', Logo: 'Lorem ipsum d', Country: 'Lorem', Industry: 'Lorem ipsum do', 'Project type': 'Lorem ipsum dolor sit amet, consect', 'System type': 'Lorem i', Platform: 'Lorem i', Architecture: 'Lorem ipsum', 'Languages and notations': '', 'IDEs and Tools': '', 'Team size': 1, Responsibilities: 'Lorem ipsum dolor sit amet, co', Role: 'Lorem ipsum dol', Position: 'Lorem ips', From: 37242, To: 38262, 'Months total': 13, 'Duration total': 'Lorem i', Reference: '', Links: 'Lorem ipsum dolor sit amet, ', Photos: 'Lorem ipsum ', Attribution: 'Lorem ipsum dolor', 'Client link': 'Lorem ipsum dolor sit amet, ', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 26, 'Project name': 'Lorem ', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore mag', Client: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod te', Logo: 'Lorem ip', Country: 'Lorem ipsum dol', Industry: 'Lorem ipsum dolor sit amet, consecte', 'Project type': 'Lorem ipsum dolor sit amet, con', 'System type': 'Lorem i', Platform: 'Lorem i', Architecture: 'Lorem ipsum dolor sit amet, cons', 'Languages and notations': 'Lorem ipsum dolor', 'IDEs and Tools': 'Lorem ipsum dolor sit a', 'Team size': 5, Responsibilities: 'Lorem ipsum dolor si', Role: 'Lorem ips', Position: 'Lorem ips', From: 35513, To: 34709, 'Months total': 12, 'Duration total': 'Lorem i', Reference: 'Lorem ', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco labori', Photos: 'Lorem ipsu', Attribution: 'Lorem ipsum dolor', 'Client link': 'Lorem ipsum dolor sit', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 27, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adi', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Client: 'Lorem ipsum dolor sit amet, consectetur adi', Logo: 'Lorem ipsum dolor sit amet', Country: 'Lorem ip', Industry: 'Lorem ipsum dolor ', 'Project type': 'Lorem ipsum dolor si', 'System type': 'Lorem ipsum ', Platform: 'Lorem ipsum dolor sit am', Architecture: 'Lorem ipsum dolor sit amet, consectetur', 'Languages and notations': 'Lorem ipsum dolo', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco', 'Team size': 2, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod', Role: 'Lorem ipsum do', Position: 'Lorem ips', From: 38289, To: 33422, 'Months total': 3, 'Duration total': 'Lorem ip', Reference: 'Lorem ipsum dolor sit ame', Links: 'Lorem ipsum dolor sit amet, con', Photos: 'Lorem ipsum dolor sit amet', Attribution: 'Lorem ipsum dolor ', 'Client link': 'Lorem ipsum dolor sit amet, con', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 28, 'Project name': 'Lorem i', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', Client: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor i', Logo: 'Lorem ipsum dolor sit amet, consectetur adipi', Country: 'Lorem ipsum dol', Industry: 'Lorem ipsum dolor sit amet, ', 'Project type': 'Lorem ipsum dol', 'System type': 'Lorem ipsum dolor sit amet, conse', Platform: 'Lorem i', Architecture: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inci', 'Languages and notations': 'Lorem ipsum dolor sit am', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim ve', 'Team size': 2, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur', Role: 'Lorem ipsum do', Position: 'Lorem ips', From: 37366, To: 37799, 'Months total': 8, 'Duration total': 'Lorem ip', Reference: 'Lorem ', Links: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna', Photos: 'Lorem ipsum', Attribution: 'Lorem ipsum dolor sit amet, consectet', 'Client link': 'Lorem ipsum dolor sit amet, consectet', Period: 'Lorem ipsum', Color: '#0000C080' }, { Id: 29, 'Project name': 'Lorem ipsum dolor sit amet, consec', Scope: 'Lorem ipsum dolor sit amet, consectetu', Client: 'Lorem ipsum dolor sit', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem ipsum', 'Project type': 'Lorem ipsum dol', 'System type': 'Lorem ipsum dolor sit amet, conse', Platform: 'Lorem i', Architecture: 'Lorem', 'Languages and notations': 'Lorem ips', 'IDEs and Tools': 'Lorem ipsum dolor sit amet,', 'Team size': 2, Responsibilities: 'Lorem ipsum do', Role: 'Lorem ipsu', Position: 'Lorem ipsu', From: 37288, To: 34617, 'Months total': 7, 'Duration total': 'Lorem ip', Reference: 'Lorem', Links: '', Photos: 'Lorem ipsum dolor sit amet, consectetu', Attribution: 'Lorem ipsum dolor sit', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ips', Color: '#C0000080' }, { Id: 30, 'Project name': 'Lorem ipsum dolor sit ame', Scope: 'Lorem ipsum dolor sit amet, consectetur ad', Client: 'Lorem ipsum dolor sit', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dol', 'System type': 'Lorem ipsum dolor sit amet, conse', Platform: 'Lorem i', Architecture: 'Lorem', 'Languages and notations': 'Lorem ips', 'IDEs and Tools': 'Lorem ipsum dolor sit amet,', 'Team size': 1, Responsibilities: 'Lorem ipsum do', Role: 'Lorem ipsu', Position: 'Lorem ipsu', From: 34573, To: 34226, 'Months total': 7, 'Duration total': 'Lorem ip', Reference: 'Lorem ip', Links: '', Photos: 'Lorem ipsum dolor sit amet, c', Attribution: 'Lorem ipsum dolor sit', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ips', Color: '#C0000080' }, { Id: 31, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', Scope: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua.', Client: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inci', Logo: 'Lorem ipsum dolor sit amet, ', Country: 'Lorem ', Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dol', 'System type': 'Lorem ipsum dolor sit amet, conse', Platform: 'Lorem ipsum dolor sit amet, con', Architecture: 'Lorem ipsum dolor sit amet, consectetur adipiscing el', 'Languages and notations': 'Lorem ips', 'IDEs and Tools': 'Lorem ipsum dolor sit amet,', 'Team size': 3, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, s', Role: 'Lorem ipsu', Position: 'Lorem ipsu', From: 35105, To: 34802, 'Months total': 23, 'Duration total': 'Lorem ipsum dolo', Reference: 'Lore', Links: 'Lorem ipsum dolor sit amet, consectetur adipi', Photos: 'Lorem ip', Attribution: 'Lorem ipsum dolor sit amet, c', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ips', Color: '#C0000080' }, { Id: 32, 'Project name': 'Lorem ipsum dolor sit amet, co', Scope: 'Lorem ipsum dolor sit amet, consec', Client: 'Lorem ipsum dolor sit', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem ipsum dolor ', 'Project type': 'Lorem ipsum dolor ', 'System type': 'Lorem i', Platform: 'Lorem i', Architecture: 'Lorem', 'Languages and notations': 'Lorem ips', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adipis', 'Team size': 1, Responsibilities: 'Lorem ipsum do', Role: 'Lorem ipsu', Position: 'Lorem ipsu', From: 37486, To: 33089, 'Months total': 4, 'Duration total': 'Lorem ip', Reference: 'Lore', Links: '', Photos: 'Lorem ipsum dolor sit amet, consec', Attribution: 'Lorem ipsum dolor sit amet, c', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ips', Color: '#C0000080' }, { Id: 33, 'Project name': 'Lorem ipsum dolor sit', Scope: 'Lorem ipsum dolor sit amet, co', Client: 'Lorem ipsum dolor sit', Logo: 'Lorem ipsum dolor sit ame', Country: 'Lorem ip', Industry: 'Lorem i', 'Project type': 'Lorem ipsum dolo', 'System type': 'Lorem ipsum dolor sit a', Platform: 'Lorem ipsum dol', Architecture: 'Lorem ipsum ', 'Languages and notations': 'Lorem ipsum dolor sit am', 'IDEs and Tools': 'Lorem ipsum dolor sit amet, consectetur adi', 'Team size': 3, Responsibilities: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod ', Role: 'Lorem ipsu', Position: 'Lorem ipsu', From: 32041, To: 31802, 'Months total': 5, 'Duration total': 'Lorem ip', Reference: 'Lore', Links: '', Photos: 'Lorem ipsum dolor sit ame', Attribution: 'Lorem ipsum dolor sit amet, c', 'Client link': 'Lorem ipsum dolor s', Period: 'Lorem ips', Color: '#C0000080' }]);

    return projects;
  }

  /**
   * Retrieves a gantt chart.
   *
   * @returns The gantt chart.
   */
  public getGanttChart(): Observable<Indexable> {
    // tslint:disable-next-line: max-line-length
    const ganttChart = of([{ Id: 1, From: 41732, To: 39516, 'From Year': 2127, 'From Month': 5, 'Project name': 'Lorem ipsum dolor sit amet, consect', Start: 1920, 'Years total': 0, Industry: 'Lorem ipsu', 'Project type': 'Lorem', Role: 'Lorem', Color: '#004000C0' }, { Id: 2, From: 39056, To: 46262, 'From Year': 2215, 'From Month': 4, 'Project name': 'Lorem ipsum dolor sit amet, consectet', Start: 1829, 'Years total': 0, Industry: 'Lorem ipsu', 'Project type': 'Lorem', Role: 'Lorem', Color: '#004000C0' }, { Id: 3, From: 40613, To: 42679, 'From Year': 2175, 'From Month': 3, 'Project name': 'Lorem ipsum dolor sit amet, consectetur', Start: 1879, 'Years total': 0, Industry: 'Lorem ipsu', 'Project type': 'Lorem', Role: 'Lorem', Color: '#004000C0' }, { Id: 4, From: 43794, To: 44703, 'From Year': 2208, 'From Month': 1, 'Project name': 'Lorem ipsum dolor sit amet, consectetu', Start: 2132, 'Years total': 0, Industry: 'Lorem ipsu', 'Project type': 'Lorem', Role: 'Lorem', Color: '#004000C0' }, { Id: 5, From: 43815, To: 46781, 'From Year': 2164, 'From Month': 13, 'Project name': 'Lorem ipsum dolor sit amet, consectetur ad', Start: 1878, 'Years total': 0, Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum dolo', Role: 'Lorem', Color: '#004000C0' }, { Id: 6, From: 45357, To: 40904, 'From Year': 1871, 'From Month': 9, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing', Start: 2045, 'Years total': 1, Industry: 'Lorem ipsu', 'Project type': 'Lor', Role: 'Lorem ipsum dolo', Color: '#00C00060' }, { Id: 7, From: 43316, To: 39377, 'From Year': 1995, 'From Month': 2, 'Project name': 'Lorem ipsum dolor sit amet, consect', Start: 2165, 'Years total': 1, Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum d', Role: 'Lorem ipsum dolo', Color: '#00C00060' }, { Id: 8, From: 37865, To: 41567, 'From Year': 1844, 'From Month': 3, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipisc', Start: 1998, 'Years total': 1, Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum', Role: 'Lorem ipsum do', Color: '#00C00060' }, { Id: 9, From: 42376, To: 42829, 'From Year': 2177, 'From Month': 3, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', Start: 2094, 'Years total': 7, Industry: 'Lorem ipsum dolor sit amet, co', 'Project type': 'Lorem ips', Role: 'Lorem ipsum do', Color: '#00C00060' }, { Id: 10, From: 41799, To: 36292, 'From Year': 2179, 'From Month': 9, 'Project name': 'Lorem ipsum dolor sit a', Start: 1876, 'Years total': 1, Industry: 'Lorem ipsum dolor sit amet, co', 'Project type': 'Lorem ips', Role: 'Lorem ipsum do', Color: '#00C00060' }, { Id: 11, From: 37640, To: 38067, 'From Year': 2103, 'From Month': 8, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipisc', Start: 2102, 'Years total': 0, Industry: 'Lorem ip', 'Project type': 'Lorem ips', Role: 'Lorem ipsum do', Color: '#00C00060' }, { Id: 12, From: 41879, To: 36664, 'From Year': 1935, 'From Month': 2, 'Project name': 'Lorem ipsum dolor sit amet, consec', Start: 2079, 'Years total': 1, Industry: 'Lorem ipsum dolor s', 'Project type': 'Lorem ipsum dolor sit amet, consectetur adipisc', Role: 'Lorem ipsum do', Color: '#00C00060' }, { Id: 13, From: 40387, To: 36678, 'From Year': 1911, 'From Month': 7, 'Project name': 'Lorem ipsum dolor sit am', Start: 1984, 'Years total': 1, Industry: 'Lorem ipsum dolor sit amet, co', 'Project type': 'Lorem ips', Role: 'Lorem ips', Color: '#00C00060' }, { Id: 14, From: 36433, To: 36895, 'From Year': 1964, 'From Month': 10, 'Project name': 'Lorem ipsum dolor sit amet, consectet', Start: 2089, 'Years total': 1, Industry: 'Lorem ipsum', 'Project type': 'Lor', Role: 'Lorem ipsum dol', Color: '#0000C080' }, { Id: 15, From: 38536, To: 38633, 'From Year': 2079, 'From Month': 4, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam,', Start: 2056, 'Years total': 2, Industry: 'Lorem ipsum dolor si', 'Project type': 'Lor', Role: 'Lorem ipsum dol', Color: '#0000C080' }, { Id: 16, From: 39282, To: 36325, 'From Year': 2027, 'From Month': 9, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod temp', Start: 1850, 'Years total': 0, Industry: 'Lorem ipsu', 'Project type': 'Lorem ipsum ', Role: 'Lorem ips', Color: '#0000C080' }, { Id: 17, From: 35644, To: 34572, 'From Year': 1922, 'From Month': 5, 'Project name': 'Lorem ipsum dolor sit amet, ', Start: 2040, 'Years total': 0, Industry: 'Lor', 'Project type': 'Lorem ipsum dolor ', Role: 'Lorem ips', Color: '#0000C080' }, { Id: 18, From: 40263, To: 38688, 'From Year': 1992, 'From Month': 5, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut ', Start: 1901, 'Years total': 1, Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dolor ', Role: 'Lorem ips', Color: '#0000C080' }, { Id: 19, From: 38005, To: 36785, 'From Year': 2159, 'From Month': 5, 'Project name': 'Lorem ipsum dolor s', Start: 2167, 'Years total': 0, Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dolo', Role: 'Lorem ipsum dolo', Color: '#0000C080' }, { Id: 20, From: 36234, To: 36397, 'From Year': 2015, 'From Month': 3, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', Start: 1909, 'Years total': 0, Industry: 'Lorem ipsum d', 'Project type': 'Lorem ipsum do', Role: 'Lorem ips', Color: '#000040C0' }, { Id: 21, From: 39590, To: 41082, 'From Year': 1816, 'From Month': 9, 'Project name': 'Lorem ipsum dolor si', Start: 2070, 'Years total': 1, Industry: 'Lor', 'Project type': 'Lorem ipsum dolor ', Role: 'Lorem ipsum dolor s', Color: '#0000C080' }, { Id: 22, From: 34426, To: 38939, 'From Year': 1961, 'From Month': 9, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tem', Start: 2187, 'Years total': 0, Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dolor ', Role: 'Lorem ipsum dolo', Color: '#0000C080' }, { Id: 23, From: 38011, To: 35091, 'From Year': 1843, 'From Month': 6, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipisc', Start: 1954, 'Years total': 0, Industry: 'Lorem ip', 'Project type': 'Lor', Role: 'Lorem ipsu', Color: '#0000C080' }, { Id: 24, From: 39858, To: 35968, 'From Year': 1913, 'From Month': 6, 'Project name': 'Lorem ipsum dolor sit amet, c', Start: 2188, 'Years total': 0, Industry: 'Lorem ipsum dolor ', 'Project type': 'Lorem ipsum dolor ', Role: 'Lorem ipsum dolo', Color: '#0000C080' }, { Id: 25, From: 35877, To: 39165, 'From Year': 1979, 'From Month': 1, 'Project name': 'Lorem ipsum dolor sit ', Start: 2182, 'Years total': 1, Industry: 'Lorem ipsum do', 'Project type': 'Lorem ipsum dolor sit amet, consect', Role: 'Lorem ipsum dol', Color: '#0000C080' }, { Id: 26, From: 38333, To: 37051, 'From Year': 2033, 'From Month': 1, 'Project name': 'Lorem ipsum dolor si', Start: 2111, 'Years total': 1, Industry: 'Lorem ipsum dolor sit amet, consecte', 'Project type': 'Lorem ipsum dolor sit amet, con', Role: 'Lorem ips', Color: '#0000C080' }, { Id: 27, From: 34912, To: 35778, 'From Year': 2037, 'From Month': 7, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, s', Start: 1855, 'Years total': 0, Industry: 'Lorem ipsum dolor ', 'Project type': 'Lorem ipsum dolor si', Role: 'Lorem ipsum do', Color: '#0000C080' }, { Id: 28, From: 32938, To: 36032, 'From Year': 2161, 'From Month': 10, 'Project name': 'Lorem ipsum dolor sit ', Start: 2172, 'Years total': 1, Industry: 'Lorem ipsum dolor sit amet, ', 'Project type': 'Lorem ipsum dol', Role: 'Lorem ipsum do', Color: '#0000C080' }, { Id: 29, From: 35862, To: 36250, 'From Year': 1952, 'From Month': 5, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscin', Start: 2022, 'Years total': 1, Industry: 'Lorem ipsum', 'Project type': 'Lorem ipsum dol', Role: 'Lorem ipsu', Color: '#C0000080' }, { Id: 30, From: 36088, To: 38775, 'From Year': 2114, 'From Month': 10, 'Project name': 'Lorem ipsum dolor sit amet, consectetur ', Start: 2107, 'Years total': 1, Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dol', Role: 'Lorem ipsu', Color: '#C0000080' }, { Id: 31, From: 32462, To: 37997, 'From Year': 1895, 'From Month': 10, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempo', Start: 2065, 'Years total': 2, Industry: 'Lorem ips', 'Project type': 'Lorem ipsum dol', Role: 'Lorem ipsu', Color: '#C0000080' }, { Id: 32, From: 38046, To: 33742, 'From Year': 2178, 'From Month': 2, 'Project name': 'Lorem ipsum dolor sit amet, consectetur adipi', Start: 1938, 'Years total': 0, Industry: 'Lorem ipsum dolor ', 'Project type': 'Lorem ipsum dolor ', Role: 'Lorem ipsu', Color: '#C0000080' }, { Id: 33, From: 35537, To: 37990, 'From Year': 2130, 'From Month': 1, 'Project name': 'Lorem ipsum dolor sit amet, consecte', Start: 2104, 'Years total': 0, Industry: 'Lorem i', 'Project type': 'Lorem ipsum dolo', Role: 'Lorem ipsu', Color: '#C0000080' }]);

    return ganttChart;
  }

  /**
   * Retrieves a general timeline.
   *
   * @returns The general timeline.
   */
  public getGeneralTimeline(): Observable<Indexable> {
    // tslint:disable-next-line: max-line-length
    const generalTimeline = of([{ Id: 1, From: 43904, To: 66259, 'From Year': 2099, 'From Month': 5, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Months total': 607, 'Duration total': 'Lor', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Start: 1953, 'Years total': 55, Type: 'Lorem ipsum d', Color: '#7F00FFC0' }, { Id: 1, From: 44158, To: 42291, 'From Year': 2020, 'From Month': 5, 'Imported Name': 'Lorem ipsum dolor sit ', 'Months total': 1, 'Duration total': 'Lorem i', Name: 'Lorem ipsum dolor sit ', Start: 1972, 'Years total': 0, Type: 'Lorem i', Color: '#004000C0' }, { Id: 2, From: 45816, To: 40343, 'From Year': 2180, 'From Month': 4, 'Imported Name': 'Lorem ipsum dolor sit am', 'Months total': 1, 'Duration total': 'Lorem i', Name: 'Lorem ipsum dolor sit am', Start: 2060, 'Years total': 0, Type: 'Lorem i', Color: '#004000C0' }, { Id: 3, From: 44665, To: 43247, 'From Year': 1840, 'From Month': 3, 'Imported Name': 'Lorem ipsum dolor sit amet', 'Months total': 1, 'Duration total': 'Lorem i', Name: 'Lorem ipsum dolor sit amet', Start: 2020, 'Years total': 0, Type: 'Lorem i', Color: '#004000C0' }, { Id: 2, From: 45222, To: 41961, 'From Year': 1878, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing ', 'Months total': 25, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing ', Start: 1935, 'Years total': 2, Type: 'Lorem ipsum d', Color: '#7F00FFC0' }, { Id: 3, From: 44198, To: 41780, 'From Year': 1993, 'From Month': 2, 'Imported Name': 'Lorem ipsum do', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum do', Start: 2191, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 4, From: 39707, To: 39526, 'From Year': 2090, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, se', Start: 2109, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 5, From: 41769, To: 39488, 'From Year': 1911, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Start: 2050, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 6, From: 39384, To: 42914, 'From Year': 2211, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit,', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit,', Start: 2058, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 7, From: 45300, To: 40247, 'From Year': 1826, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consecte', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consecte', Start: 1944, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 8, From: 41002, To: 43869, 'From Year': 1916, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit', Start: 2077, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 9, From: 42029, To: 46333, 'From Year': 2128, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectet', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectet', Start: 1894, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 10, From: 42534, To: 47473, 'From Year': 1880, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor s', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor s', Start: 1855, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 11, From: 40587, To: 43053, 'From Year': 1932, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectet', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectet', Start: 1915, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 12, From: 44404, To: 45902, 'From Year': 1950, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, con', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, con', Start: 2095, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 13, From: 44806, To: 39501, 'From Year': 2201, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur ', Start: 1950, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 14, From: 41079, To: 40210, 'From Year': 2116, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur ', Start: 1882, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 15, From: 44026, To: 44750, 'From Year': 1871, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipisc', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipisc', Start: 2050, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 16, From: 46783, To: 45657, 'From Year': 1836, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit amet, consecte', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consecte', Start: 1857, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 17, From: 43711, To: 45470, 'From Year': 2194, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit amet, consect', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consect', Start: 2014, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 18, From: 40525, To: 39532, 'From Year': 2107, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit ', Start: 2152, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 19, From: 42213, To: 41125, 'From Year': 2168, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipi', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipi', Start: 2159, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 4, From: 41949, To: 46710, 'From Year': 2167, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit am', 'Months total': 3, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit am', Start: 1903, 'Years total': 0, Type: 'Lorem i', Color: '#004000C0' }, { Id: 5, From: 46169, To: 40742, 'From Year': 2167, 'From Month': 12, 'Imported Name': 'Lorem ipsum dolor sit amet, c', 'Months total': 1, 'Duration total': 'Lorem i', Name: 'Lorem ipsum dolor sit amet, c', Start: 2208, 'Years total': 0, Type: 'Lorem i', Color: '#004000C0' }, { Id: 20, From: 43247, To: 44842, 'From Year': 2088, 'From Month': 11, 'Imported Name': 'Lorem ipsum do', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum do', Start: 1965, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 21, From: 46882, To: 38850, 'From Year': 1826, 'From Month': 10, 'Imported Name': 'Lorem ipsum ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum ', Start: 2045, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 22, From: 44799, To: 45703, 'From Year': 2066, 'From Month': 10, 'Imported Name': 'Lorem ipsum dolo', 'Months total': 6, 'Duration total': 'L', Name: 'Lorem ipsum dolo', Start: 2143, 'Years total': 1, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 23, From: 43297, To: 38973, 'From Year': 2140, 'From Month': 7, 'Imported Name': 'Lorem ipsu', 'Months total': 11, 'Duration total': 'Lo', Name: 'Lorem ipsu', Start: 1954, 'Years total': 1, Type: 'Lorem ipsum do', Color: '#FF800080' }, { Id: 6, From: 40835, To: 40207, 'From Year': 1888, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consecte', 'Months total': 8, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consecte', Start: 1960, 'Years total': 1, Type: 'Lorem i', Color: '#00C00060' }, { Id: 7, From: 42573, To: 39616, 'From Year': 1921, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit', 'Months total': 8, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit', Start: 2042, 'Years total': 1, Type: 'Lorem i', Color: '#00C00060' }, { Id: 8, From: 38313, To: 40189, 'From Year': 2003, 'From Month': 3, 'Imported Name': 'Lorem ipsum dolor sit amet, cons', 'Months total': 12, 'Duration total': 'Lorem ips', Name: 'Lorem ipsum dolor sit amet, cons', Start: 1886, 'Years total': 1, Type: 'Lorem i', Color: '#00C00060' }, { Id: 24, From: 37530, To: 43929, 'From Year': 2092, 'From Month': 7, 'Imported Name': 'Lorem ipsum dolor sit amet, ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, ', Start: 1915, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 9, From: 43303, To: 43892, 'From Year': 1810, 'From Month': 3, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur ', 'Months total': 78, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consectetur ', Start: 1938, 'Years total': 7, Type: 'Lorem i', Color: '#00C00060' }, { Id: 10, From: 39006, To: 41604, 'From Year': 1822, 'From Month': 8, 'Imported Name': 'Lorem i', 'Months total': 11, 'Duration total': 'Lorem ips', Name: 'Lorem i', Start: 1835, 'Years total': 1, Type: 'Lorem i', Color: '#00C00060' }, { Id: 11, From: 38234, To: 42434, 'From Year': 1995, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit amet, cons', 'Months total': 6, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, cons', Start: 2028, 'Years total': 0, Type: 'Lorem i', Color: '#00C00060' }, { Id: 25, From: 37838, To: 35948, 'From Year': 2048, 'From Month': 3, 'Imported Name': 'Lorem ipsum dolor sit amet, c', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, c', Start: 2171, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 12, From: 42246, To: 40606, 'From Year': 1845, 'From Month': 2, 'Imported Name': 'Lorem ipsum ', 'Months total': 19, 'Duration total': 'Lorem ipsum dol', Name: 'Lorem ipsum ', Start: 2013, 'Years total': 1, Type: 'Lorem i', Color: '#00C00060' }, { Id: 13, From: 40651, To: 40167, 'From Year': 2118, 'From Month': 6, 'Imported Name': 'Lorem ips', 'Months total': 8, 'Duration total': 'Lorem ip', Name: 'Lorem ips', Start: 2199, 'Years total': 1, Type: 'Lorem i', Color: '#00C00060' }, { Id: 14, From: 37710, To: 37190, 'From Year': 2019, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit ', 'Months total': 8, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit ', Start: 2041, 'Years total': 1, Type: 'Lorem i', Color: '#0000C080' }, { Id: 15, From: 37102, To: 35216, 'From Year': 1966, 'From Month': 4, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut', 'Months total': 20, 'Duration total': 'Lorem ipsum dol', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut', Start: 2078, 'Years total': 2, Type: 'Lorem i', Color: '#0000C080' }, { Id: 16, From: 40340, To: 38454, 'From Year': 2094, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', 'Months total': 3, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed ', Start: 2107, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 17, From: 36258, To: 36904, 'From Year': 1854, 'From Month': 6, 'Imported Name': 'Lorem ipsum d', 'Months total': 3, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum d', Start: 2080, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 19, From: 36056, To: 38917, 'From Year': 2029, 'From Month': 5, 'Imported Name': 'Lorem', 'Months total': 1, 'Duration total': 'Lorem i', Name: 'Lorem', Start: 2041, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 18, From: 38586, To: 38417, 'From Year': 1971, 'From Month': 5, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempo', 'Months total': 10, 'Duration total': 'Lorem ips', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempo', Start: 1810, 'Years total': 1, Type: 'Lorem i', Color: '#0000C080' }, { Id: 20, From: 40971, To: 37782, 'From Year': 2153, 'From Month': 3, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adip', 'Months total': 2, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consectetur adip', Start: 1843, 'Years total': 0, Type: 'Lorem i', Color: '#000040C0' }, { Id: 22, From: 40553, To: 33883, 'From Year': 1993, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed', 'Months total': 4, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed', Start: 1900, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 21, From: 33920, To: 37576, 'From Year': 2162, 'From Month': 9, 'Imported Name': 'Lorem', 'Months total': 7, 'Duration total': 'Lorem ip', Name: 'Lorem', Start: 2197, 'Years total': 1, Type: 'Lorem i', Color: '#0000C080' }, { Id: 23, From: 36580, To: 38173, 'From Year': 1907, 'From Month': 7, 'Imported Name': 'Lorem ipsum dolor sit amet, cons', 'Months total': 2, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, cons', Start: 1975, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 24, From: 38960, To: 37552, 'From Year': 2035, 'From Month': 6, 'Imported Name': 'Lorem ipsum dol', 'Months total': 1, 'Duration total': 'Lorem i', Name: 'Lorem ipsum dol', Start: 2151, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 25, From: 39696, To: 38156, 'From Year': 1987, 'From Month': 1, 'Imported Name': 'Lorem ip', 'Months total': 12, 'Duration total': 'Lorem i', Name: 'Lorem ip', Start: 1950, 'Years total': 1, Type: 'Lorem i', Color: '#0000C080' }, { Id: 26, From: 40098, To: 39551, 'From Year': 1904, 'From Month': 1, 'Imported Name': 'Lorem ', 'Months total': 13, 'Duration total': 'Lorem i', Name: 'Lorem ', Start: 2120, 'Years total': 1, Type: 'Lorem i', Color: '#0000C080' }, { Id: 27, From: 34927, To: 35166, 'From Year': 2040, 'From Month': 7, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adi', 'Months total': 3, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consectetur adi', Start: 1953, 'Years total': 0, Type: 'Lorem i', Color: '#0000C080' }, { Id: 28, From: 36568, To: 33479, 'From Year': 1919, 'From Month': 10, 'Imported Name': 'Lorem i', 'Months total': 8, 'Duration total': 'Lorem ip', Name: 'Lorem i', Start: 1922, 'Years total': 1, Type: 'Lorem i', Color: '#0000C080' }, { Id: 26, From: 36818, To: 34152, 'From Year': 2032, 'From Month': 7, 'Imported Name': 'Lorem ipsum dolor sit amet, con', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, con', Start: 1989, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 27, From: 34125, To: 38342, 'From Year': 1855, 'From Month': 5, 'Imported Name': 'Lorem ipsum do', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum do', Start: 2058, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 28, From: 34385, To: 36721, 'From Year': 1960, 'From Month': 3, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipisci', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipisci', Start: 2103, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 29, From: 38321, To: 37485, 'From Year': 1946, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetu', 'Months total': 2, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetu', Start: 2176, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 30, From: 34692, To: 35344, 'From Year': 2191, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur', Start: 1880, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 31, From: 35058, To: 32991, 'From Year': 1924, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, co', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, co', Start: 1934, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 32, From: 38906, To: 39179, 'From Year': 1884, 'From Month': 10, 'Imported Name': 'Lorem ipsum dolor sit amet, ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, ', Start: 1918, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 33, From: 38842, To: 33820, 'From Year': 2100, 'From Month': 10, 'Imported Name': 'Lorem ipsum do', 'Months total': 2, 'Duration total': 'L', Name: 'Lorem ipsum do', Start: 1947, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 34, From: 35754, To: 38473, 'From Year': 2119, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor si', 'Months total': 4, 'Duration total': 'L', Name: 'Lorem ipsum dolor si', Start: 1890, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 35, From: 39396, To: 34312, 'From Year': 2166, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit amet, conse', 'Months total': 8, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, conse', Start: 2149, 'Years total': 1, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 1, From: 35176, To: 34574, 'From Year': 1918, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit amet, con', 'Months total': 10, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, con', Start: 2054, 'Years total': 1, Type: 'Lorem ips', Color: '#008080C0' }, { Id: 36, From: 37050, To: 38136, 'From Year': 1923, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipisci', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipisci', Start: 2169, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 29, From: 34666, To: 37743, 'From Year': 1863, 'From Month': 5, 'Imported Name': 'Lorem ipsum dolor sit amet, consec', 'Months total': 7, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, consec', Start: 1874, 'Years total': 1, Type: 'Lorem i', Color: '#C0000080' }, { Id: 30, From: 34132, To: 38576, 'From Year': 1900, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit ame', 'Months total': 7, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit ame', Start: 2192, 'Years total': 1, Type: 'Lorem i', Color: '#C0000080' }, { Id: 31, From: 32349, To: 37742, 'From Year': 2049, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', 'Months total': 22, 'Duration total': 'Lorem ipsum dolo', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing eli', Start: 1823, 'Years total': 2, Type: 'Lorem i', Color: '#C0000080' }, { Id: 1, From: 32111, To: 34717, 'From Year': 2121, 'From Month': 5, 'Imported Name': 'Lorem ipsum dolor si', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor si', Start: 2090, 'Years total': 0, Type: 'Lorem ipsum', Color: '#00FFFFC0' }, { Id: 37, From: 35320, To: 34999, 'From Year': 2022, 'From Month': 11, 'Imported Name': 'Lorem ipsum dol', 'Months total': 21, 'Duration total': 'Lo', Name: 'Lorem ipsum dol', Start: 2158, 'Years total': 2, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 2, From: 35653, To: 37440, 'From Year': 2051, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, c', 'Months total': 68, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, c', Start: 2049, 'Years total': 6, Type: 'Lorem ips', Color: '#008080C0' }, { Id: 32, From: 37255, To: 32829, 'From Year': 2066, 'From Month': 2, 'Imported Name': 'Lorem ipsum dolor sit amet, co', 'Months total': 4, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit amet, co', Start: 2116, 'Years total': 0, Type: 'Lorem i', Color: '#C0000080' }, { Id: 33, From: 31657, To: 36479, 'From Year': 2106, 'From Month': 1, 'Imported Name': 'Lorem ipsum dolor sit', 'Months total': 6, 'Duration total': 'Lorem ip', Name: 'Lorem ipsum dolor sit', Start: 2026, 'Years total': 0, Type: 'Lorem i', Color: '#C0000080' }, { Id: 1, From: 35451, To: 41151, 'From Year': 2092, 'From Month': 12, 'Imported Name': 'Lorem ipsum dolor sit amet, conse', 'Months total': 271, 'Duration total': 'Lor', Name: 'Lorem ipsum dolor sit amet, conse', Start: 1832, 'Years total': 21, Type: 'Lorem ipsu', Color: '#FF800080' }, { Id: 2, From: 32003, To: 34001, 'From Year': 1838, 'From Month': 5, 'Imported Name': 'Lorem ipsum dol', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dol', Start: 1955, 'Years total': 0, Type: 'Lorem ipsum', Color: '#00FFFFC0' }, { Id: 38, From: 32752, To: 30890, 'From Year': 2098, 'From Month': 5, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inc', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor inc', Start: 1926, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 39, From: 36622, To: 35385, 'From Year': 2148, 'From Month': 1, 'Imported Name': 'Lorem ipsum dol', 'Months total': 12, 'Duration total': 'Lo', Name: 'Lorem ipsum dol', Start: 1982, 'Years total': 1, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 40, From: 30980, To: 31879, 'From Year': 1944, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, ', 'Months total': 19, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, ', Start: 2048, 'Years total': 2, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 41, From: 30848, To: 30936, 'From Year': 1876, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, ', 'Months total': 29, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, ', Start: 1913, 'Years total': 3, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 2, From: 32973, To: 30173, 'From Year': 2073, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consect', 'Months total': 21, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, consect', Start: 2034, 'Years total': 2, Type: 'Lorem ipsu', Color: '#FF800080' }, { Id: 3, From: 33470, To: 36338, 'From Year': 2169, 'From Month': 10, 'Imported Name': 'Lorem ipsum dolor sit amet, consectet', 'Months total': 58, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, consectet', Start: 1947, 'Years total': 6, Type: 'Lorem ips', Color: '#008080C0' }, { Id: 42, From: 31620, To: 30269, 'From Year': 2041, 'From Month': 7, 'Imported Name': 'Lorem ipsum dolor sit amet, cons', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, cons', Start: 2072, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 43, From: 28804, To: 33211, 'From Year': 1803, 'From Month': 6, 'Imported Name': 'Lorem ipsum dolor sit ', 'Months total': 1, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit ', Start: 2030, 'Years total': 0, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 44, From: 30486, To: 29478, 'From Year': 2117, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit ', 'Months total': 32, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit ', Start: 1879, 'Years total': 3, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 45, From: 32040, To: 32253, 'From Year': 2063, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit', 'Months total': 35, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit', Start: 1935, 'Years total': 3, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 46, From: 28792, To: 35439, 'From Year': 1837, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit ', 'Months total': 57, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit ', Start: 2062, 'Years total': 5, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 4, From: 29695, To: 34141, 'From Year': 1994, 'From Month': 9, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adi', 'Months total': 51, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, consectetur adi', Start: 1808, 'Years total': 5, Type: 'Lorem ips', Color: '#008080C0' }, { Id: 47, From: 32499, To: 32663, 'From Year': 1923, 'From Month': 10, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore ', 'Months total': 7, 'Duration total': 'L', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore ', Start: 2017, 'Years total': 1, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 48, From: 29213, To: 30979, 'From Year': 2051, 'From Month': 10, 'Imported Name': 'Lorem ipsum do', 'Months total': 51, 'Duration total': 'Lo', Name: 'Lorem ipsum do', Start: 2002, 'Years total': 4, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 49, From: 27792, To: 31266, 'From Year': 1840, 'From Month': 10, 'Imported Name': 'Lorem ipsum dolo', 'Months total': 54, 'Duration total': 'Lo', Name: 'Lorem ipsum dolo', Start: 1824, 'Years total': 4, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 50, From: 28910, To: 32026, 'From Year': 1892, 'From Month': 10, 'Imported Name': 'Lorem ipsum d', 'Months total': 81, 'Duration total': 'Lo', Name: 'Lorem ipsum d', Start: 2134, 'Years total': 7, Type: 'Lorem ipsum do', Color: '#C000C080' }, { Id: 5, From: 30920, To: 30029, 'From Year': 2039, 'From Month': 8, 'Imported Name': 'Lorem ipsum dolor sit amet, consectetur adipiscin', 'Months total': 86, 'Duration total': 'Lo', Name: 'Lorem ipsum dolor sit amet, consectetur adipiscin', Start: 2037, 'Years total': 7, Type: 'Lorem ips', Color: '#008080C0' }]);

    return generalTimeline;
  }

  /**
   * Retrieves an entities array.
   *
   * @returns The entities array.
   */
  public getEntities(): Observable<Indexable<any>> {
    // tslint:disable-next-line: max-line-length
    const entities = of({ 'Curriculum Vitae': { node: 'Lorem ipsum dolo', parent: '', class: 'Lorem', main: 'Lore' }, 'Personal Data': { node: 'Lorem ipsum d', parent: '', class: 'Lorem' }, Background: { node: 'Lorem ipsu', parent: '', class: 'Lorem' }, 'Professional Experience': { node: 'Lorem ipsu', parent: '', class: 'Lore' }, Education: { node: 'Lorem ips', parent: '', class: 'Lore' }, Accomplishments: { node: 'Lorem ipsum dol', parent: '', class: 'Lorem' }, Certifications: { node: 'Lorem ipsum do', parent: '', class: 'Lore' }, Languages: { node: 'Lorem ips', parent: '', class: 'Lore' }, Courses: { node: 'Lorem i', parent: '', class: 'Lore' }, 'Courses Index': { node: 'Lorem ipsum d', parent: '', class: 'Lore' }, 'Courses List': { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, Organizations: { node: 'Lorem i', parent: '', class: 'Lore' }, 'Organizations Index': { node: 'Lorem ipsum d', parent: '', class: 'Lore' }, 'Organizations List': { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, Volunteering: { node: 'Lorem i', parent: '', class: 'Lore' }, 'Volunteering Index': { node: 'Lorem ipsum d', parent: '', class: 'Lore' }, 'Volunteering List': { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, Vacation: { node: 'Lorem i', parent: '', class: 'Lore' }, 'Vacation Index': { node: 'Lorem ipsum d', parent: '', class: 'Lore' }, 'Vacation List': { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, Publications: { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, 'Publications Index': { node: 'Lorem ipsum dolor ', parent: '', class: 'Lore' }, 'Publications List': { node: 'Lorem ipsum dolor', parent: '', class: 'Lore' }, 'Project Summary': { node: 'Lorem ipsum dol', parent: '', class: 'Lorem', main: 'Lore' }, 'Areas of Expertise': { node: 'Lorem ipsum dolor ', parent: 'Lorem ipsum dol', class: 'Lorem' }, Client: { node: 'Lorem ', parent: 'Lorem ipsum dolor ', class: 'Lore' }, Country: { node: 'Lorem ', parent: 'Lorem ipsum dolor ', class: 'Lore' }, 'Country Index': { node: 'Lorem ', parent: 'Lorem ipsum dolor ', class: 'Lore' }, 'Country Map': { node: 'Lorem ', parent: 'Lorem ipsum dolor ', class: 'Lore' }, Industry: { node: 'Lorem ip', parent: 'Lorem ipsum dolor ', class: 'Lore' }, 'Project type': { node: 'Lorem ipsum ', parent: 'Lorem ipsum dolor ', class: 'Lore' }, 'System type': { node: 'Lorem ipsum', parent: 'Lorem ipsum dolor ', class: 'Lore' }, Skills: { node: 'Lorem ', parent: 'Lorem ipsum dol', class: 'Lorem' }, Platform: { node: 'Lorem ip', parent: 'Lorem ', class: 'Lore' }, Architecture: { node: 'Lorem ipsum ', parent: 'Lorem ', class: 'Lore' }, 'Languages and notations': { node: 'Lorem ipsum dolor sit a', parent: 'Lorem ', class: 'Lore' }, 'IDEs and Tools': { node: 'Lorem ipsum do', parent: 'Lorem ', class: 'Lore' }, 'Job Functions': { node: 'Lorem ipsum d', parent: 'Lorem ipsum dol', class: 'Lorem' }, Responsibilities: { node: 'Lorem ipsum dolo', parent: 'Lorem ipsum d', class: 'Lore' }, Role: { node: 'Lore', parent: 'Lorem ipsum d', class: 'Lore' }, 'Team size': { node: 'Lorem ips', parent: 'Lorem ipsum d', class: 'Lore' }, Position: { node: 'Lorem ip', parent: 'Lorem ipsum d', class: 'Lore' }, Reference: { node: 'Lorem ips', parent: 'Lorem ipsum d', class: 'Lore' }, 'Project Portfolio': { node: 'Lorem ipsum dolor', parent: '', class: 'Lorem', main: 'Lore' }, 'Gantt Chart': { node: 'Lorem ipsum', parent: '', class: 'Lorem' }, List: { node: 'Lore', parent: '', class: 'Lorem' }, Contributions: { node: 'Lore', parent: '', class: 'Lorem' }, Index: { node: 'Lorem', parent: '', class: 'Lorem' }, Projects: { node: 'Lorem ip', parent: 'Lorem ipsum dolor', class: 'Lorem' }, 'Modern Age': { node: 'Lorem ipsu', parent: 'Lorem ip', class: 'Lore' }, Renaissance: { node: 'Lorem ipsum', parent: 'Lorem ip', class: 'Lore' }, 'Dark Ages': { node: 'Lorem ips', parent: 'Lorem ip', class: 'Lore' }, 'General Timeline': { node: 'Lorem ipsum dolo', parent: '', class: 'Lorem', main: 'Lore' }, Id: { node: 'Id', parent: '', class: '' }, From: { node: 'Lore', parent: '', class: '' }, To: { node: 'Lo', parent: '', class: '' }, 'Project name': { node: 'Lorem ipsum ', parent: '', class: '' }, Links: { node: 'Lorem', parent: '', class: '' }, References: { node: 'Lorem ipsu', parent: '', class: '' }, Pipeline: { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, 'Service catalog': { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, Version: { node: 'Lorem ipsum ', parent: '', class: 'Lore' }, Navigation: { node: 'Lorem ipsum ', parent: '', class: 'Lore' } });

    return entities;
  }

  /**
   * Retrieves a UI.
   *
   * @returns The UI.
   */
  public getUi(): Observable<Indexable<UiEntry>> {
    const ui = of(this.mockUi);
    return ui;
  }

  /**
   * Retrieves a Version.
   *
   * @returns The Version.
   */
  public getVersion(): Observable<{ builds: { version: string }[] }> {
    const version = of({ builds: [{ version: '1.2.3' }] });
    return version;
  }
}
